module Isomorfeus
  class AssetManager
    class Asset
      attr_reader :bundle, :bundle_size
      attr_reader :bundle_gz, :bundle_gz_size
      attr_reader :bundle_map
      attr_reader :bundle_map_gz
      attr_reader :mtime, :ruby_imports, :target
      attr_reader :mutex

      def initialize(target = :browser)
        @mutex = Mutex.new
        raise "Unknown asset target!" unless %i[browser node].include?(target)
        @target = target
        @bundled = false
        @ruby_compiled = false
        @js_imports = []
        @ruby_imports = []
      end

      def add_js_import(*args)
        @js_imports << Isomorfeus::AssetManager::JsImport.new(*args)
      end

      def add_ruby_import(*args)
        @ruby_imports << Isomorfeus::AssetManager::RubyImport.new(*args)
      end

      def bundle=(b)  
        @bundled = true
        @bundle = b
        @bundle_size = @bundle.size
        unless @target == :node
          @bundle_gz = Zlib::gzip(b, level: Zlib::BEST_COMPRESSION) 
          @bundle_gz_size = @bundle_gz.size
        end
        @bundle
      end

      def bundle_map=(m)  
        @bundled = true
        @bundle_map = m
        @bundle_map_gz = Zlib::gzip(m, level: Zlib::BEST_COMPRESSION) unless @target == :node
        @bundle_map
      end

      def bundled?
        @bundled
      end

      def ruby_modules
        @ruby_imports.map(&:module_name)
      end

      def touch
        @mtime = Time.now
      end

      def to_s
        js = @target == :node ? '' : "if (typeof globalThis !== 'undefined' && typeof global === 'undefined') { globalThis.global = globalThis; }\n"
        unless @js_imports.empty?
          js << "#{@js_imports.map(&:to_s).join("\n")}"
        end
        js << "\n" if !@js_imports.empty? && !@ruby_imports.empty?
        unless @ruby_imports.empty?
          if Isomorfeus.development? && @target == :browser
            js << <<~JAVASCRIPT
            // Isomorfeus Asset Manager HMR code begin
            let ws_protocol = (window.location.protocol == 'https:') ? 'wss:' : 'ws:';
            let ws_url = ws_protocol + '//' + window.location.host + "#{Isomorfeus.assets_websocket_path}";
            let hmr_ws = new WebSocket(ws_url);
            hmr_ws.onmessage = function(event) {
              let update = JSON.parse(event.data);
              if (typeof update.error !== 'undefined') { console.error(update.error); return; }
              let start_index = '/* Generated by Opal 1.2.0 */\\nOpal.modules[\\"'.length;
              let end_index = update.javascript.indexOf('"', start_index);
              let opal_module_name = update.javascript.substr(start_index, end_index - start_index);
              console.log('Updating ', opal_module_name);
              if (typeof Opal !== 'undefined' && typeof Opal.require_table !== "undefined" && Opal.require_table['corelib/module']) {
                try {
                  eval(update.javascript);
                  if (Opal.require_table[opal_module_name]) { Opal.load.call(Opal, opal_module_name); } 
                  else { Opal.require.call(Opal, opal_module_name); }
                  Opal.Isomorfeus.$force_render();
                } catch (e) { console.error(e); return; }
              }
            }
            // Isomorfeus Asset Manager HMR code end
            JAVASCRIPT
          end
          js << "#{@ruby_imports.map(&:to_s).join("\n")}"
        end
        js
      end
    end
  end
end
