module Isomorfeus
  # available settings
  if RUBY_ENGINE == 'opal'
    add_client_option(:assets_path)
  else
    class << self
      attr_reader :env
      attr_accessor :root
      attr_accessor :app_root
      attr_accessor :assets_path
      attr_accessor :assets_websocket_path 
      attr_accessor :asset_manager_tmpdir
      attr_accessor :asset_manager_hmr_channel
      attr_accessor :asset_manager_hmr_dirs
      attr_accessor :node_paths
      attr_accessor :assets
      attr_accessor :hmr_listener

      def add_web_js_import(*args)
        Isomorfeus.assets['web.js'].add_js_import(*args)
      end
  
      def add_ssr_js_import(*args)
        Isomorfeus.assets['ssr.js'].add_js_import(*args)
      end
  
      def add_common_js_import(*args)
        Isomorfeus.assets['web.js'].add_js_import(*args)
        Isomorfeus.assets['ssr.js'].add_js_import(*args)
      end
  
      def add_web_ruby_import(*args)
        Isomorfeus.assets['web.js'].add_ruby_import(*args)
      end
  
      def add_ssr_ruby_import(*args)
        Isomorfeus.assets['ssr.js'].add_ruby_import(*args)
      end
  
      def add_common_ruby_import(*args)
        Isomorfeus.assets['web.js'].add_ruby_import(*args)
        Isomorfeus.assets['ssr.js'].add_ruby_import(*args)
      end

      # client side env is initialized in isomorfeus-preact
      def env=(env_string)
        @env = env_string ? env_string.to_s : 'development'
        @development = (@env == 'development') ? true : false
        @production = (@env == 'production') ? true : false
        @test = (@env == 'test') ? true : false
      end

      def development?
        @development
      end

      def production?
        @production
      end

      def test?
        @test
      end
    end
  end

  self.hmr_listener = nil
  self.asset_manager_hmr_channel = :isomorfeus_asset_manager_module_updates
  self.asset_manager_hmr_dirs = %w[channels, components, data, operations, policies]
  self.assets_websocket_path = '/_assets_websocket'
  self.assets_path = '/assets'
  self.assets = {
    'web.js' => Isomorfeus::AssetManager::Asset.new(:browser),
    'ssr.js' => Isomorfeus::AssetManager::Asset.new(:node)
  }
  self.asset_manager_tmpdir = Dir.mktmpdir
  Kernel.at_exit { FileUtils.rm_rf(Isomorfeus.asset_manager_tmpdir) if Dir.exist?(Isomorfeus.asset_manager_tmpdir) }
end
